clear all;
close all;
clc;

%% Initial value

% The functions
f = {@(x)(0.5*((6*x - 2).^2).*sin(12.*x - 4) + 10.*(x - 0.5));
    @(x)( ((6*x - 2).^2).*sin(12.*x - 4) + 10)};

% Input domain
bnd = stk_hrect([0; 1], {'x'});

% The noise
noise_std = [0, 0];
is_noise = any(noise_std ~= 0);

% The cost of each level
cost_lev = [1, 4];

% The critical threshold
thres = 10;

% Reference
reg = 2.5;
range_out = 20;

% Number of points in the initial design
ni_init = fliplr(cumprod([3, 2]));


dim = size(bnd, 2);
nb_lev = length(f);

% Assert cost are sorted
cost_lev = sort(cost_lev);
supp_budget = 9*max(cost_lev);

% Real function
nt = 978;
x  = stk_sampling_regulargrid(nt, dim, bnd);
dx = mean(diff(x));
xt = stk_add_level(x, 1:nb_lev);

zt = stk_dataframe(NaN(nt*nb_lev, 1), {'z'});
for kl = 1:nb_lev
    zt((kl - 1)*nt + (1:nt), 1) = f{kl}(x);
end

idx_nb_lev = (double(xt(:, dim + 1)) == nb_lev);
pt = stk_distrib_normal_cdf(double(zt(idx_nb_lev, 1)), thres, noise_std(nb_lev));

% Number of parameters
nb_sim_par = 100;
% nb_sim_par_surCrit = 1: The MAP parameters

% Parameters of the Haario algorithm
time_no_adapt = 500;
burn_in       = 1000;
delay         = 100;

type_adapt      = 'smooth';
display_waitbar = true;

% Parameter of the Sequential Monte-Carlo sampling
nb_step = 20;

% Integration point (for criterion)
xu = xt(idx_nb_lev, :);
epsilon = 5e-4; % Zero crit

% Maximum number of evalutations
maxEval = 20*dim;
opts_fminbnd = optimset('fminbnd');
opts_fminbnd = optimset(opts_fminbnd, 'Display', 'off', 'MaxFunEvals', maxEval);


%% Style plot

% Noise
color_noise = [0.25, 0.75, 0.3];	% Green-Cyan
style_noise = {color_noise, 'LineStyle', '-', 'LineWidth', 1,...
    'EdgeColor', color_noise};

% Uncertainty
color_unc = [0.8, 0.8, 0.8];
style_unc = {color_unc, 'LineStyle', '-', 'LineWidth', 1,...
    'EdgeColor', color_unc};

% Prediction
style_pred  = {'LineStyle', '-', 'LineWidth', 3, 'Color', [0.95 0.25 0.3]};


% Real function
color_lev = [0, 0.5, 0;
    0, 0, 0.5];
color_truth = [0.39, 0.47, 0.64];
style_truth = {'LineStyle', '--', 'LineWidth', 3};

% Observations
style_obs   = {'LineStyle', 'none', 'Marker', 'o', 'MarkerSize', 10,...
    'Color', 'k', 'MarkerFaceColor', 'k'};

% Threshold
style_thres =  {'LineStyle', '-.', 'LineWidth', 2, 'Color', 'k'};

% Best reduction
style_best = {'LineStyle', 'none', 'Marker', 's', 'MarkerSize', 10,...
    'Color', 'm', 'MarkerFaceColor', 'none'};

% Criterion
style_crit = {'LineStyle', '-', 'LineWidth', 3, 'Marker', 'none'};

% Saving folder
foldersave = '';
% fullfile(pwd, '..', 'redactions', 'presentations', 'these_juin2018', 'images', '3_planification');
% fullfile(pwd, '..', 'redactions', 'rapports', 'these', 'images', 'plan_seq');
nb_fig = 6;
delta_cost = supp_budget/(nb_fig - 1);

% Style

ftz_axi = 20;
ftz_lab = 24;
ftz_tit = 28;
ftz_leg = 20;
tick_input = double(bnd(1)):0.2:double(bnd(2));

kunc = -sqrt(2)*erfcinv(1 + 0.95);

%% Create a model

% Covariance
cov_ar = stk_covmodel_autoregre(nb_lev, dim, reg);

% Prior of the covariance parameters
if isnan(reg)
    reg = 2.5;
    var_reg = log(5/2);
else
    var_reg = 0;
end

mean_prior = [ones(nb_lev - 1, 1);
    repmat([2*log(range_out/100); -log(diff(bnd)/2)'; log(reg);], nb_lev, 1)];
var_prior = [repmat(2, nb_lev - 1, 1);
    repmat([log(100); repmat(log(10), dim, 1); var_reg;], nb_lev, 1)].^2;

cov_ar.prior = stk_prior_gauss(mean_prior, var_prior);


% Regularization
if is_noise
    noivar = stk_noisemodel_multilev(1:2, NaN(2, 1), true,...
        2*log(range_out/100), 0.01*log(100).^2, 0.99*log(100).^2);
    % Fixed parameters
    is_fixed = ([var_prior; diag(noivar.prior.covariance)] == 0);
else
    lnv_regul = 2*log(1e-4);
    % Fixed parameters
    is_fixed = [(var_prior == 0); true];
end

% Model
model = stk_model(cov_ar.func, dim + 1);
model.param = cov_ar;
if is_noise
    model.lognoisevariance = noivar;
else
    model.lognoisevariance = lnv_regul;
end
% model.lm = stk_lm_constant();
model.lm = @(x)(stk_cokriging_mean(1:nb_lev, x));

%% Initial design
% Observation
xi_init = stk_sampling_nestedlhs(ni_init, dim, bnd, [], 1:nb_lev);

zi_init = stk_dataframe(NaN(size(xi_init, 1), 1), {'z'});
for id_l = 1:nb_lev
    idx_il = (xi_init.Level == id_l);
    zi_init(idx_il) = f{id_l}(double(xi_init(idx_il, 1:dim)))...
        + noise_std(id_l)*randn(sum(idx_il), 1);
end

% Initial model
if is_noise
    [param_init, lnv_init] = stk_param_init(model, xi_init, zi_init, [bnd, [1; nb_lev]], true);
    
    [param_init, lnv_init] = stk_param_estim (model, xi_init, zi_init,...
        param_init, lnv_init, @stk_param_mlogpost);
    model.param = param_init;
    model.lognoisevariance = lnv_init;
else
    param_init = stk_param_init(model, xi_init, zi_init, [bnd, [1; nb_lev]], false);
    
    param_init = stk_param_estim (model, xi_init, zi_init,...
        param_init, [], @stk_param_mlogpost);
    model.param = param_init;
end

% The fixed parameters
param_vect = stk_get_optimizable_parameters(model);

param_fixed = NaN(size(param_vect));
param_fixed(is_fixed) = param_vect(is_fixed);
% Seed of the adaptive metropolis
sim_par0 = param_vect(~is_fixed, 1);

% The distribution of parameters
distrib = @(p)(stk_haario_logpost(model, xi_init, zi_init, p, param_fixed));
isLogDistrib = true;

% Box of parameters
[lb_cov, ub_cov] = stk_param_getdefaultbounds(model.param, xi_init, zi_init);
if is_noise
    [lb_lnv, ub_lnv] = stk_param_getdefaultbounds(model.lognoisevariance, xi_init, zi_init);
    bnd_param = [lb_cov', lb_lnv'; ub_cov', ub_lnv'];
else
    bnd_param = [lb_cov'; ub_cov'];
end
bnd_param = bnd_param(:, ~is_fixed);

% Initial covariance of parameters
cov_par0 = stk_haario_suggest_C0(distrib, bnd_param,...
    sim_par0, isLogDistrib);

% Haario's sampling
sim_par_init = stk_haario_adaptive_metropolis(distrib, ...
    bnd_param, nb_sim_par, sim_par0, cov_par0,...
    time_no_adapt, burn_in, delay, isLogDistrib,...
    [], type_adapt, display_waitbar);

% Simulated parameters
simul_param_init = repmat(param_fixed', nb_sim_par, 1);
simul_param_init(:, ~is_fixed) = sim_par_init;

%% Loop of algorithm
% Prepare design
xi = xi_init;
zi = zi_init;

ni = ni_init;

cost_init = sum(cost_lev.*ni_init);
tot_budget = cost_init + supp_budget;
% Prepare model
model.param             = param_init;
simul_param             = simul_param_init;

% Prepare other informations
% New points
xn = [];
zn = [];
nn = zeros(1, nb_lev);
% Predictions
zp = stk_dataframe(NaN(nb_lev*nt, 3), {'mean', 'var', 'var_obs'});
pp = stk_dataframe(NaN(nt, 2), {'mean', 'var'});

next_fig = 0;

%%
fig_pred = NaN(nb_lev, 1);
fig_conf = NaN;
fig_crit = NaN;
lim_pred = NaN(2, nb_lev);
while sum((ni + nn).*cost_lev) <= tot_budget
    
    %% Add the new points
    xi = cat(1, xi, xn);
    zi = cat(1, zi, zn);
    ni = ni + nn;
    consumed_budget = sum(ni.*cost_lev) - cost_init;
    fprintf('Cost: %d/%d (%.1f%%)\n',...
        consumed_budget, supp_budget, 100*consumed_budget/supp_budget);
    
    %% Reestimate parameters
    disp('Updating hyper-parameters...');
    
    if is_noise
        [param_opt, lnv_opt] = stk_param_estim (model, xi, zi,...
            model.param, model.lognoisevariance, @stk_param_mlogpost);
        model.param = param_opt;
        model.lognoisevariance = lnv_opt;
    else
        param_opt = stk_param_estim (model, xi, zi, model.param, [], @stk_param_mlogpost);
        model.param = param_opt;
    end
    
    % Display estimated parameters
    for ul = 1:(nb_lev - 1)
        fprintf('rho_opt[%d] = %g\n', ul, model.param.param.rhoCoeff(ul));
    end
    for pl = 1:nb_lev
        fprintf('Level %d:\n', pl);
        fprintf('      Sigma  = %g\n', model.param.param.(['sig_', num2str(pl)]));
        for kd = 1:dim
            fprintf('      Rho[%d] = %g\n', kd, model.param.param.(['range_', num2str(pl)]));
        end
        fprintf('      Nu     = %g\n', model.param.param.(['reg_', num2str(pl)]));
        if is_noise
            fprintf('sqrt(lambda) = %g\n', model.lognoisevariance.param.noistd(pl));
        end
    end
    
    % The vector of hyper-parameters
    vect_estim = stk_get_optimizable_parameters(model);
    param_center = vect_estim(~is_fixed, 1);
    % Move the parameters
    if isempty(xn) % first loop
        SMC_weights = ones(nb_sim_par, 1)/nb_sim_par;
    else
        ni_x = size(xi, 1);
        iniM = 1:(ni_x - 1);
        % SMC - 1: update weights
        SMC_weights = stk_smc_weights(simul_param, model,...
            xi(iniM, :), zi(iniM, :), xi(ni_x, :), zi(ni_x, :), SMC_weights);
        % SMC - 2: re-sample
        [simul_param, SMC_weights] = stk_smc_sample(simul_param, SMC_weights);
    end
    % SMC - 3: move
    % Select varying parameters
    sim_par = simul_param(:, ~is_fixed);
    % The distribution of parameters
    distrib = @(p)(stk_haario_logpost(model, xi, zi, p, param_fixed));
    
    % Box of parameters
    [lb_cov, ub_cov] = stk_param_getdefaultbounds(param_opt, xi, zi);
    
    if is_noise
        [lb_lnv, ub_lnv] = stk_param_getdefaultbounds(lnv_opt, xi, zi);
        bnd_param = [lb_cov', lb_lnv'; ub_cov', ub_lnv'];
    else
        bnd_param = [lb_cov'; ub_cov'];
    end
    bnd_param = bnd_param(:, ~is_fixed);
    
    % Covariance for parameters
    cov_param = stk_haario_suggest_C0(distrib, bnd_param, param_center, true);
    var_param = diag(cov_param);
    too_close = (var_param./max(var_param) < sqrt(eps));
    var_param(too_close) = diff(bnd_param(:, too_close)).^2*0.01;
    cov_param = diag(var_param);
    
    sim_par = stk_smc_move(sim_par, distrib, nb_step, cov_param);
    % Simulated parameters
    simul_param = repmat(param_fixed', nb_sim_par, 1);
    simul_param(:, ~is_fixed) = sim_par;
    
    disp('Updating hyper-parameters achived.');
    %% Prediction
    disp('Predicting...');
    t_prediction = tic;
    % Data for prediction
    pred_mean_mean   = NaN(nt*nb_lev, 1);
    pred_mean_var    = NaN(nt*nb_lev, 1);
    pred_var_mean    = NaN(nt*nb_lev, 1);
    pred_mean_varobs = NaN(nt*nb_lev, 1);
    
    
    % Data for probability estimations
    fail_mean_mean = NaN(nt, 1);
    fail_mean_var  = NaN(nt, 1);
    fail_var_mean  = NaN(nt, 1);
    
    % Check the MAP parameter
    best_AP = stk_param_mlogpost(model, xi, zi);
    best_cov = param_opt;
    if is_noise
        best_lnv = lnv_opt;
    end
    reestimate = false;
    
    for kpar = 1:nb_sim_par
        % 1 - change the parameters
        model = stk_set_optimizable_parameters(model, simul_param(kpar, :));
        
        % Bonus - check posterior
        AP = stk_param_mlogpost(model, xi, zi);
        if AP <= best_AP
            best_AP = AP;
            best_cov = model.param;
            if is_noise
                best_lnv = model.lognoisevariance;
            end
            reestimate = true;
        end
        
        % 2 - compute the prediction, knowing the parameters
        zp_k = stk_predict(model, xi, zi, xt);
        if isa(model.lognoisevariance, 'stk_noisemodel')
            var_obs_k = stk_covmat_noise (model, xt, [], -1, true);
        else % Homoscedastic
            var_obs_k = exp(model.lognoisevariance);
        end
        zp_k_var_obs = zp_k.var + var_obs_k;
        
        % 3 - the local probability of failure, knowing the parameters
        zp_k_lev = zp_k(idx_nb_lev, :);
        zp_k_lev_var_obs = zp_k_var_obs(idx_nb_lev, :);
        up_k_lev = (zp_k_lev.mean - thres)./sqrt(zp_k_lev_var_obs);
        
        if is_noise
            rp_k_lev = zp_k_lev.var./zp_k_lev_var_obs;
            
            pp_mean = stk_distrib_normal_cdf(up_k_lev);
            
            fail_k_mean = pp_mean;
            fail_k_var = stk_distrib_bivnorm_cdf([up_k_lev, up_k_lev],...
                0, 0, 1, 1, rp_k_lev);
            fail_k_var = fail_k_var - pp_mean.^2;
        else
            [pp_mean, pq_mean] = stk_distrib_normal_cdf(up_k_lev);
            
            fail_k_mean = pp_mean;
            fail_k_var  = pp_mean.*pq_mean;
        end
        
        % 4 - mean over the parameters (reccursive formulation)
        pred_mean_var = stk_mean_recursive(kpar, pred_mean_var, zp_k.var);
        pred_mean_varobs = stk_mean_recursive(kpar, pred_mean_varobs, zp_k_var_obs);
        
        [pred_mean_mean, pred_var_mean] = stk_update_recursive(kpar, pred_mean_mean,...
            pred_var_mean, zp_k.mean);
        
        % probability of failure
        fail_mean_var = stk_mean_recursive(kpar, fail_mean_var, fail_k_var);
        
        [fail_mean_mean, fail_var_mean] = stk_update_recursive(kpar, fail_mean_mean,...
            fail_var_mean, fail_k_mean);
    end
    zp.mean     = pred_mean_mean;
    zp.var      = pred_var_mean + pred_mean_var;
    zp.var_obs  = pred_var_mean + pred_mean_varobs;
    
    pp.mean	= fail_mean_mean;
    pp.var  = fail_mean_var + fail_var_mean;
    
    if reestimate
        disp('Reestimation');
        if is_noise
            [param_opt, lnv_opt] = stk_param_estim(model, xi, zi, best_cov,...
                best_lnv, @stk_param_mlogpost);
        else
            param_opt = stk_param_estim(model, xi, zi, best_cov,...
                [], @stk_param_mlogpost);
        end
    end
    model.param = param_opt;
    if is_noise
        model.lognoisevariance = lnv_opt;
    end
    
    disp('Predicting achieved.');
    %% Compute criteria
    disp('Selecting a new observation...');
    t_select_new_point = tic;
    
    max_red = NaN(nb_lev, 1);
    x_best  = NaN(nb_lev, dim + 1);
    
    % MRSUR
    [~, red] = stk_sur_pexceedthres(thres, xu, model, xi, zi, xt);
    
    msur    = NaN(nb_lev, 1);
    is_free = true(nb_lev, 1);
    for kl = 1:nb_lev
        idtm = (double(xt(:, dim + 1)) == kl);
        redm = red(idtm);
        % Remove points which are to close to the
        % observations
        if ~stk_isnoisy(model)
            idim = (double(xi(:, dim + 1)) == kl);
            
            distm = min(stk_dist(xi(idim, 1:dim), xt(idtm, 1:dim)));
            
            redm(distm < epsilon) = 0;
            red(idtm) = redm;
        end
        
        % maximal reduction at this level
        [~, id_red] = max(redm);
        
        % Optimize a little
        func_opt_k = @(x)(stk_sur_pexceedthres(thres, xu,...
            model, xi, zi, stk_add_level(x, kl)));
        xini_ko = x(id_red, :);
        lower_bnd = double(max(double(bnd(1, :)), xini_ko - dx));
        upper_bnd = double(min(double(bnd(2, :)) , xini_ko + dx));
        
        x_opt_k = fminbnd(func_opt_k, lower_bnd, upper_bnd, opts_fminbnd);
        
        x_best(kl, :) = [x_opt_k, kl];
        [~, max_red(kl)] = stk_sur_pexceedthres(thres, xu,...
            model, xi, zi, x_best(kl, :));
        
        msur(kl) = max_red(kl)/cost_lev(kl);
        % Is the level blocked
        % is_free(kl) = ((consumed_budget + cost_lev(kl)) <= supp_budget);
    end
    
    if all(~is_free)  % if the budget is reached
        is_free = true(nb_lev, 1);
        % Select which you want
    end
    
    msur = msur.*double(is_free);   % 0 if the level is not free
    
    [~, id_lev] = max(msur);
    xn = x_best(id_lev, :);
    zn = f{id_lev}(double(xn(1, 1:dim))) + noise_std(id_lev)*randn(1, 1);
    
    nn = zeros(1, nb_lev);
    nn(id_lev) = 1;
    
    fprintf('\nNew point: (x = %g, t = %d)\n', xn(1, 1), xn(1, 2));
    
    disp('Selecting a new observation achieved.');
    %% Plot real data
    ncol = 3;
    
    for kl = nb_lev:-1:1
        if ishandle(fig_pred(kl))
            figure(fig_pred(kl));
        else
            fig_pred(kl) = stk_figure();
        end
        set(fig_pred(kl), 'name', ['Prediction at s = ', num2str(kl),...
            ' with n = [', num2str(ni),...
            '] points (Total = ', num2str(sum(ni)), '). Cost = ', num2str(consumed_budget)]);
        % Functions
        hold off;
        
        idx_k = (double(xt(:, dim + 1)) == kl);
        idi_k = (double(xi(:, dim + 1)) == kl);
        zp_k = zp(idx_k, :);
        
        if is_noise
            % Plot noise part
            delta_noise = kunc * sqrt (abs (zp_k.var_obs));
            fill([double(x); flipud(double(x))],...
                [zp_k.mean - delta_noise;
                flipud(zp_k.mean + delta_noise)], style_noise{:});
            hold on;
        end
        % Plot the uncertainty part
        delta = kunc * sqrt (abs (zp_k.var));
        fill([double(x); flipud(double(x))],...
            [zp_k.mean - delta; flipud(zp_k.mean + delta)], style_unc{:});
        hold on;
        % The prediction
        plot(double(x), double(zp_k.mean), style_pred{:});
        
        % The true function
        plot(double(x), double(zt(idx_k, 1)), style_truth{:}, 'Color', color_truth);
        hold on;
        % Observations
        plot(double(xi(idi_k, 1:dim)), double(zi(idi_k, 1)), style_obs{:});
        
        if xn(:, dim + 1) == kl
            hn = plot(double(xn(1, 1)), zn, style_best{:});
            set(hn, 'MarkerFaceColor', 'm');
        end
        if kl == nb_lev
            % Threshold
            plot(double(bnd), repmat(thres, 2, 1), style_thres{:});
        end
        
        xlim(double(bnd));
        if all(isnan(lim_pred(:, kl)))
            lim_pred(:, kl) = get(gca, 'YLim');
        end
        ylim(lim_pred(:, kl));
        set(gca, 'FontSize', ftz_axi, 'XTick', tick_input);
        stk_xlabel('x', 'FontSize', ftz_lab);
        stk_ylabel(['Z_{x,', num2str(kl), '}'], 'FontSize', ftz_lab);
        stk_title(sprintf('Niveau %d (n=[%d,%d] => C=%d)',...
            kl, ni(1), ni(2), consumed_budget), 'FontSize', ftz_tit);
    end
    
    % Probability of threshold
    if ishandle(fig_conf)
        figure(fig_conf);
    else
        fig_conf = stk_figure();
    end
    set(fig_conf, 'name', ['Probabilities with n = [', num2str(ni),...
        '] points (Total = ', num2str(sum(ni)), '). Cost = ', num2str(consumed_budget)]);
    hold off;
    % Plot the uncertainty part
    delta_pr = kunc * sqrt (abs (pp.var));
    fill([double(x); flipud(double(x))],...
        [pp.mean - delta_pr; flipud(pp.mean + delta_pr)], style_unc{:});
    hold on;
    % The true function
    plot(double(x), double(pt), style_truth{:}, 'Color', color_truth);
    % The prediction
    plot(double(x), double(pp.mean), style_pred{:});
    % Threshold
    plot(double(bnd), repmat(0.5, 2, 1), style_thres{:});
    xlim(double(bnd));
    ylim([0; 1]);
    set(gca, 'FontSize', ftz_axi, 'XTick', tick_input);
    stk_xlabel('x', 'FontSize', ftz_lab);
    stk_ylabel(['P(Z_2 > ', num2str(thres), '|x, \chi_{[', num2str(ni(1)),...
        ',', num2str(ni(2)), ']})'], 'FontSize', ftz_lab);
    stk_title('Probabilit� de d�faillance', 'FontSize', ftz_tit);
    
    % Reduction of uncertainty
    if ishandle(fig_crit)
        figure(fig_crit);
    else
        fig_crit = stk_figure();
    end
    set(fig_crit, 'name', ['Criterion with n = [', num2str(ni),...
        '] points (Total = ', num2str(sum(ni)), '). Cost = ', num2str(consumed_budget)]);
    hold off;
    for kc = nb_lev:-1:1
        
        idc_k = (double(xt(:, dim + 1)) == kc);
        
        xc_k = xt(idc_k, 1:dim);
        red_k = red(idc_k, 1);
        
        plot(double(xc_k), double(red_k)./cost_lev(kc), style_crit{:}, 'Color', color_lev(kc, :));
        hold on;
        hm = plot(double(x_best(kc, 1:dim)), max_red(kc)./cost_lev(kc), style_best{:});
        if xn(:, dim + 1) == kc
            set(hm, 'MarkerFaceColor', 'm');
        end
        
    end
    xlim(double(bnd));
    ylim([0; 1.1*max(max_red(:)./cost_lev(:))]);
    set(gca, 'FontSize', ftz_axi, 'XTick', tick_input);
    stk_xlabel('x', 'FontSize', ftz_lab);
    stk_ylabel(['G_{[', num2str(ni(1)), ',', num2str(ni(2)), ']}(x,\delta)/C(\delta)'], 'FontSize', ftz_lab);
    stk_title('Crit�re d''�chantillonnage', 'FontSize', ftz_tit);
    
    % Save figures
    if sum(ni) - sum(ni_init) <= 5
        save_fig(foldersave,...
            ['plan2S_pred1_C', num2str(consumed_budget)], fig_pred(1));
        save_fig(foldersave,...
            ['plan2S_pred2_C', num2str(consumed_budget)], fig_pred(2));
        save_fig(foldersave,...
            ['plan2S_prob_C', num2str(consumed_budget)], fig_conf);
        save_fig(foldersave,...
            ['plan2S_crit_C', num2str(consumed_budget)], fig_crit);
    end
    
    if consumed_budget >= next_fig
        save_fig(foldersave,...
            ['plan2S_pred1_C', num2str(consumed_budget)], fig_pred(1));
        save_fig(foldersave,...
            ['plan2S_pred2_C', num2str(consumed_budget)], fig_pred(2));
        save_fig(foldersave,...
            ['plan2S_prob_C', num2str(consumed_budget)], fig_conf);
        save_fig(foldersave,...
            ['plan2S_crit_C', num2str(consumed_budget)], fig_crit);
        
        next_fig = next_fig + delta_cost;
    end
    
    drawnow;
    fprintf('\n');
    
end

%% Plot legend

stk_figure;
set(gcf, 'name', 'Legend');
hold off;
if is_noise
    hnoi = fill([1, 2, 2, 1], [1, 2, 2, 1], style_noise{:});
    leg_noi = {'Incertitudes sur les observations'};
else
    hnoi = [];
    leg_noi = {};
end
hold on;
hunc = fill([1, 2, 2, 1], [1, 2, 2, 1], style_unc{:});
htru = plot([1, 2], [1, 2], style_truth{:}, 'Color', color_truth);
hpre = plot([1, 2], [1, 2], style_pred{:});
htre = plot([1, 2], [1, 2], style_thres{:});
hobs = plot([1, 2], [1, 2], style_obs{:});
hnew = plot([1, 2], [1, 2], style_best{:}, 'MarkerFaceColor', 'm');
xlim([-2, -1]);
ylim([-2, -1]);
set(gca, 'XTick', [], 'YTick', [], 'XColor', 'w', 'YColor', 'w');

hleg = legend([hobs, htre, hpre, hunc, hnoi, hnew, htru],...
    'Observations', 'Seuil critique', 'Moyenne a posteriori',...
    'Incertitudes sur la moyenne', leg_noi{:},...
    'Future observation', 'Vraie fonction', 'Orientation', 'Vertical');

set(hleg, 'FontSize', ftz_leg);
pos_leg = get(hleg, 'Position');
pos_leg(1:2) = 0.5 - pos_leg(3:4)/2;
set(hleg, 'Position', pos_leg);

% Reduction of uncertainty
stk_figure();
set(gcf, 'name', 'Legend of criterion');
hold off;
hcrit = NaN(nb_lev, 1);
for kc = nb_lev:-1:1
    
    if kc == nb_lev
        typlev = 'haute-fid�lit�';
    elseif kc == 1
        typlev = 'basse-fid�lit�';
    else
        typlev = 'interm�diaire';
    end
    
    hcrit(kc) = plot([1, 2], [1, 2], style_crit{:}, 'Color', color_lev(kc, :));
    legcrit{kc} = ['Crit�re au niveau ', typlev, ' (s = ', num2str(kc), ')'];
    hold on;
    hm = plot([1, 2], [1, 2], style_best{:});
    if xn(:, dim + 1) == kc
        set(hm, 'MarkerFaceColor', 'm');
    end
    
end
xlim([-2, -1]);
ylim([-2, -1]);
set(gca, 'XTick', [], 'YTick', [], 'XColor', 'w', 'YColor', 'w');

hleg = legend(hcrit, legcrit{:}, 'Orientation', 'Vertical');

set(hleg, 'FontSize', ftz_leg);
pos_leg = get(hleg, 'Position');
pos_leg(1:2) = 0.5 - pos_leg(3:4)/2;
set(hleg, 'Position', pos_leg);