function y = F(x)
% F is the multi-fidelity functions of Forrester
%
% CALL: Y = F (X)
% 
%   computes the multi-fidelity functions defined on [0; 1].
%   f (X) = { f1 (X(:, 1)), if X(:, 2) = 1
%           { F2 (X(:, 1)), if X(:, 2) = 2
%           { error, otherwise
%
% REFERENCE
%
%   [1] Alexander I. J. Forrester, Andrs Sobester, and Andy J. Keane.
%   Multifidelity optimization via surrogate modelling. Proceedings of
%   the Royal Society of London A: Mathematical, Physical and Engineering
%   Sciences, 463(2088):32513269, 2007.

if nargout > 1
    error('Too many input arguments')
end

if size(x, 2) ~= 2
    error('The inputs must be two-dimensional')
end

x = double(x);
b1 = (x(:, end) == 1);
b2 = (x(:, end) == 2);
if any(~b1 & ~b2)
    error('The levels must be equal to 1 or 2');
end

y = zeros(size(x, 1), 1);
y(b1) = F1(x(b1, 1));
y(b2) = F2(x(b2, 1));

end

