function y = F2(x)
% F2 is the high-fidelity function of Forrester
%
% CALL: Y = F2 (X)
% 
%   computes the low-fidelity function defined on [0; 1].
%   F2 (X) = 2*F1 (X) - 20.*x + 20 = ((6*x - 2).^2).*sin(12.*x - 4) + 10;
%
% REFERENCE
%
%   [1] Alexander I. J. Forrester, Andrs Sobester, and Andy J. Keane.
%   Multifidelity optimization via surrogate modelling. Proceedings of
%   the Royal Society of London A: Mathematical, Physical and Engineering
%   Sciences, 463(2088):32513269, 2007.

if size(x, 2) > 1
    error('The inputs must be one-dimensional')
end

x = double(x);

% y = 2.*(0.5*((6*x - 2).^2).*sin(12.*x - 4) + 10.*(x - 0.5)) - 20.*x + 20;
y = ((6*x - 2).^2).*sin(12.*x - 4) + 10;
end

