% This file proposes an example of simulation and prediction with the
% Haario algorithm.

config_init;    % Function, observations, configurations

% Model with prior and MaP parameters
model = set_model_init(covariance, xi, zi, bnd, range_z, is_noise);

%% Samples according to the posterior distribution using Haario's algorithm

% Rules the behavior of the algorithm
t0 = 200;   % burn-in 'without adaptation'
t1 = 500;   % burn-in 'with adaptation'
t2 = 20;    % Delay between two returned parameters
updating_formula = 'hard';  % Available: 'hard', 'smooth' or 'hastings'

% Sampling
ts = tic;
[sim_param, fig_corr] = sampling_haario(nb_param, model, xi, zi, t0, t1, t2, updating_formula);
time_simulate = toc(ts);
fprintf('Time for simulating hyper-parameters: %.0f s = %d min %.0f s\n',...
    time_simulate, floor(time_simulate/60), time_simulate - 60*floor(time_simulate/60) );

%% Plot density of parameters
fig_par = plot_density_parameters(sim_param, model, sig_noise, nd);
  
%% Make predictions
% Predictions taking into account on simulated parameters
tp = tic;
zp = stk_haario_predict(sim_param, model, xi, zi, xt);
time_predict = toc(tp);
fprintf('Time for predicting: %.2f s\n', time_predict);

%% Figure
stk_figure('Results with Haario Metropolis adaptive algorithm');
plot1d_noise ([], xi, zi, xt, zt, zp);
% Bounds
xlim(bnd);
% Labels
stk_xlabel('x');
stk_ylabel('Z');
stk_title('Prediction using Haario adaptive algorithm');