% STK_MEAN_RECURSIVE computes the mean with a recursive formula
%
% CALL : M_n = stk_mean_recursive(N, M_nM1, X_n)
%
%   computes the mean M_n = mean([X_1, ...,X_nM1 , X_n]),
%   using the number N, the previous mean M_nM1 = mean([X_1, ..., X_nM1]),
%   and the new value X_n.
%
% See also: stk_update_recursive, stk_mean_recursive
function m_n = stk_mean_recursive(n, m_nM1, x_n)

if nargin ~= 3
    if nargin < 3
        stk_error('Not enough input arguments', 'NotEnoughInputArgs');
    else
        stk_error('Too many input arguments', 'TooManyInputArgs');
    end
end

%% Compute the mean
[n, m_nM1, x_n] = stk_commonsize(n, m_nM1, x_n);

b0 = (n <= 0);
b1 = (n == 1);

% n > 1
m_n = ((n - 1)./n).*m_nM1 + x_n./n;
% n == 1
m_n(b1) = x_n(b1);  % n == 1 ==> mean([x_1, ..., x_n]) = mean([x_1]) = x_1
% n < 1
m_n(b0) = NaN;   % n <= 0 ==> mean([x_1, ... , x_n]) = mean([]) = NaN

end

%!shared n, x, m
%! x = [0.92, 0.98, 0.36, -2.93, -0.67, -0.81, 0.19, -0.93, 0.13, -2.87];
%! n = length(x);
%! m = 0.75;

%!error m = stk_mean_recursive();
%!error m = stk_mean_recursive(n);
%!error m = stk_mean_recursive(n, m);
%!test  m = stk_mean_recursive(n, m, x(n));
%!error m = stk_mean_recursive(n, m, x(n), pi);

%!test
%! m = nan;
%! for k = 1:n
%!  m = stk_mean_recursive(k, m, x(k));
%!  assert( stk_isequal_tolabs(m, mean(x(1:k))) );
%! end

%! m = mean(x);
%!assert( isnan(stk_mean_recursive(0, m, x(n))) );
%!assert( isequal(x, stk_mean_recursive(1, m, x)) );