% CALL : str = char(promo)
%
% This function transform a STK_PROTOMODEL to a string (a matrix of char).
% Use by 'disp' function.
function str = char(promo)

str_func  = ' func: ';
str_param = 'param: ';
str_prior = 'prior: ';
% create the 3 fields of a stk_protomodel
blank = repmat(' ', 1, 7);
% 8 = length(str_func) = length(str_param) = length(str_prior)

%% Function
func = promo.func;

if isempty(func) %remove the stupid case
    name_func = '''''';     % appear as ''
elseif ischar(func)   % if func is a string
    name_func = func;
else % if ~ischar(func)
    try
        name_func = char(func);   % try to convert in char
    catch
        name_func = ['<', class(func), '>'];
    end
end

func_part = [[str_func; repmat(blank, size(name_func, 1) - 1, 1)], name_func];

%% Parameters
param = promo.param;

if isempty(param);
    name_param = '[]';
elseif isnumeric(param) % first case
    name_param = num2str(param);
else % second case : isa(param, 'stk_param');
    try
        name_param = char(param);   % try to convert in char
    catch
        name_param = ['<', class(param), '>'];
    end
end

param_part = [[str_param; repmat(blank, size(name_param, 1) - 1, 1)], name_param];

%% Prior distribution
prior = promo.prior;

if isempty(prior);
    name_prior = '[]';
elseif isstruct(prior) % first case
    name_prior = loc_struct2str(prior);
else % second case : isa(prior, 'stk_prior');
    try
        name_prior = char(prior);   % try to convert in char
    catch
        name_prior = ['<', class(prior), '>'];
    end
end

prior_part = [[str_prior; repmat(blank, size(name_prior, 1) - 1, 1)], name_prior];

%% Final string
str = char(func_part, param_part, prior_part);
end

%% Convert structure to string
function str = loc_struct2str(struc)

%% Name of properties to display
fields = fieldnames(struc);             %the fields of structures
nameLength = zeros(length(fields), 1);	%the lenght of each fields names
maxNameLength = 0;                      %the maximum lenght of names

for kn = 1:length(nameLength)
    nameLength(kn, 1) = length(fields{kn, 1});
    if nameLength(kn, 1) > maxNameLength
        maxNameLength = nameLength(kn, 1);
    end
end%get back the longest property name length

blank = repmat(' ', 1, maxNameLength + 2);

%% Write properties
stringToDisplay = '';   %first line, to initialize

for kf = 1:length(fields)
    field = struc.(fields{kf, 1});
    field_name = [repmat(' ', 1, maxNameLength - nameLength(kf, 1)),...
        fields{kf, 1}, ': '];%complete the name by empty caracheter, to align the ':'
    
    switch class(field)
        case 'char'
            charProp = field;
            
        case 'double'
            charProp = num2str(field);  %convert num  (matrix) to string
            
        case 'logical'
            charProp = num2str(field);  %convert bool (matrix) to string
            charProp = cellstr(charProp);
            charProp = strrep(charProp, '0', 'false');  % change 0 by false
            charProp = strrep(charProp, '1', 'true ');  %    and 1 by true
            charProp = char(charProp);
            
        otherwise
            str_size = strrep(num2str(size(field)), '  ', 'x');
            charProp = ['<', str_size, ' ', class(field), '>'];
            % Default method : plot size and class
    end
    
    field_name = [ [field_name; repmat(blank, size(charProp, 1) - 1, 1)], charProp];
    stringToDisplay = char (stringToDisplay, field_name);%vertical concatenation
end

stringToDisplay = stringToDisplay(2:(size(stringToDisplay, 1)), :);
%delete the first line, which is empty

str = stringToDisplay;
end
