% CALL : promo = set(promo, propname, value)
%
% Change STK_PROTOMODEL properties.
function promo = set(promo, propname, value)

switch propname
    case 'func'
        
        if isempty(value)
            stk_error('''func'' cannot be empty. You have to define your function.',...
                'InvalidArgument');
        end
        
        if isa(value, 'function_handle')
            value = func2str(value);  % convert to string
        end
        promo.func = value;
        
    case 'param'
        if isempty(value)   
            value = []; % assert the type of empty object
        end
        
        if ~isempty(value) && ~isnumeric(value) && ~isa(value, 'stk_param')
            % tolerate: empty parameters; numeric parameters; and stk_param classes.
            stk_error(['A parameter of a stk_protomodel',...
                ' must be a numeric or a stk_param.'], 'InvalidArgument');
        end

        promo.param = value;
        
    case 'prior'
        
        if isempty(value)
            value = []; % assert the type of empty object
        end
        
        if ~isempty(value) && ~isstruct(value) && ~isa(value, 'stk_prior')
            % tolerate: empty priors; structure; and stk_prior classes.
            stk_error(['A prior log-distribution of a stk_protomodel',...
                ' must be a structure or a stk_prior.'], 'InvalidArgument');
        end
        
        promo.prior = value;
        
    otherwise
        if ischar(propname)
            stk_error(['A stk_protomodel does not have any property named ',...
                propname], 'UnknownField');
        else
            stk_error('''propname'' must be a char.', 'InvalidArgument');
        end
end
end

