% CALL : covmo = stk_covmodel(func, param, prior)
%
% The constructor of the class covariance model.
% - func : a function handle, or a char, or a class,
% which is the covariance function.
% Must support the inputs : feval(func, param, xi, yi, diff, pariwise);
% - param : parameter of the covariance;
% - prior : the prior distribution.
function covmo = stk_covmodel(func, param, prior)

% Default values
switch nargin
    case 0
        func = 'stk_materncov52_iso';
        param = NaN(2, 1);
        prior = [];
    case 1
        param = [];
        prior = [];
    case 2
        prior = [];
    case 3
        % do nothing
    otherwise
        stk_error('Too many input arguments', 'TooManyInputArgs');
end

%% Build object
covmo = struct();
covmo = class(covmo, 'stk_covmodel', stk_protomodel(func, param, prior));

end

