% CALL : covmo_matiso = stk_covmodel_materniso(reg)
%
% Create a covariance model with a Matern isotropic covariance function.
%
% - reg (default value : NaN) : the regularity of the function.
function covmo_matiso = stk_covmodel_materniso(reg)

%% Default value
if nargin < 1 || isempty(reg)
    reg = NaN;
else
    if reg < 0
        stk_error('Regularity must be a positive parameter.', 'InvalidArgument');
    end
end

%% Function and parameters
if isinf(reg)
    func = 'stk_gausscov_iso';
    param = stk_param_iso(NaN, NaN);
else
    func = 'stk_materncov_iso';
    param = stk_param_matern_iso(NaN, NaN, log(reg));
end


%% Prior
prior = [];

%% Create the covariance model.
covmo_matiso = stk_covmodel(func, param, prior);
covmo_matiso = class(struct, 'stk_covmodel_materniso', covmo_matiso);
end

