function [lb, ub] = stk_brownian_cov_getdefaultbounds(param0, ti, zi)
%UNTITLED Summary of this function goes here
%   Detailed explanation goes here

if nargin > 3
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

if nargin < 3
    stk_error('Not enough input arguments.', 'NotEnoughInputArgs');
end

%% Check sizes

dim = size(ti, 2);
if dim ~= 1
    stk_error('ti must be a vector of dimension 1.', 'InvalidArgument');
end

ni = size(ti, 1);
if ~isequal(size(zi), [ni, 1])
    stk_error ('zi should be a column, with the same number of rows as xi.',...
        'IncorrectSize');
end

if numel(param0) < 2
    stk_error('Param0 must have length upper than 2.', 'InvalidArgument');
elseif numel(param0) > 3
    stk_error('Param0 must have length lower than 3.', 'InvalidArgument');
end

%% Initialize bounds
% constants
opts = stk_options_get ('stk_param_getdefaultbounds');
TOLVAR = opts.tolvar;

% bounds for the variance parameter
log_empirical_variance = log (var (double (zi)));
if isinf(log_empirical_variance)
    logvar_lb = param0(1) - TOLVAR;
    logvar_ub = param0(1) + TOLVAR;
else
    logvar_lb = min (log_empirical_variance, param0(1)) - TOLVAR;
    logvar_ub = max (log_empirical_variance, param0(1)) + TOLVAR;
end

% bounds for the exponential parameters
L_lb = min (log(0.1), param0(2));
L_ub = max (log (10), param0(2));

lb = [logvar_lb; L_lb;];
ub = [logvar_ub; L_ub;];
% If there is an intercept parameters, set the intercept parameters
if numel(param0) == 3
    lb = [lb; param0(3)];
    ub = [ub; param0(3)];
end

end