function mean_vec = stk_cokriging_mean(levels, x)
% STK_COKRIGING_MEAN computes the mean of a cokriging model
%
% CALL: MEAN_VEC = stk_cokriging_mean(LEVELS, X)
%
% 	return a matrix whose size is size(X, 1) x length(LEVELS),
% 	whose value were in {0; 1}, with mean (i, j) = 1 if and only
%	if x(i, dim + 1) = valueLevel(j).
%	Model for ordinary cokriging, with constant mean at each level.

if nargin > 2,
    stk_error ('Too many input arguments.', 'TooManyInputArgs');
end

levels = levels(:); % assert column vector

x = double (x);
nb = size(x, 1);

len_levels = length(levels);

x_level = x(:, end);
mean_vec = NaN(nb, len_levels);
% save the number of level, corresponding to the level

for kl = 1:len_levels
    mean_vec(:, kl) = (x_level == levels(kl));
end
% Assert double
mean_vec = double(mean_vec);

end