% CALL : kov = stk_covmat (noimo, x1, x2)
%
% CALL : kov = stk_covmat (noimo, x1, x2, diff)
%
% CALL : kov = stk_covmat (noimo, x1, x2, diff, pairwise)
%
% Calculate the value of the noise variance matrix/vector.
%
% - noimo : a noise variance model;
% - xi : some points where noise variance must be compute
% - diff (default value : -1) : ask for a derivative of noise variance
% according to the diff-th parameter of the noise variance model;
% - pairwise : (default value : false) a boolean indicating if output 'kov'
% must be a vector of size [ni, 1], or a diagonal matrix of size [ni, ni];
function kov = stk_covmat (noimo, x1, x2, diff, pairwise)

if nargin < 5 || isempty (pairwise)
    pairwise = false;
end

if nargin < 4 || isempty (diff)
    diff = -1;
end

assert (isempty (x2));

kov = feval (get (noimo, 'func'), get(noimo, 'param'), x1, diff);

% If pairwise, return result as matrix
if ~ pairwise
    kov = diag (kov);
end

end

