% CALL : mf_cov = set(ar_cov, propname, value)
% [overload stk_param function]
function ar_cov = set(ar_cov, propname, value)

switch propname
    case 'rhoCoeff'
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''rhoCoeff'' must be a double.', 'InvalidType');
            end
        end
        value = value(:);   % assert column vector
        nb_lev = number_levels(ar_cov);
        if length(value) ~= (nb_lev - 1)
            stk_error('The length of the new rho vector must be the same as the old one',...
                'IncorrectSize');
        end
        ar_cov.rhoCoeff = value;
        
    case 'params'
        if ~iscell(value)
            stk_error('''params'' must be a cell array of parameters',...
                'InvalidArgument');
        end
        
        value = value(:);   %assert column cells
        nb_lev = number_levels(ar_cov);
        if length(value) ~= nb_lev
            stk_error('The new cell of parameters must have the same lenght as the old one',...
                'IncorrectSize');
        end
        
        if any(cellfun(@(x)(~isa(x, 'stk_param')), value))
            stk_error('Parameters must be covariance parameters', 'InvalidArgument');
        end
        ar_cov.params = value;
        
        
        %% Other propositions : change a property of one of the 3 parameters
    otherwise
        id_underscore = strfind(propname, '_');
        if isempty(id_underscore)
            stk_error('Unknown property', 'UnknownProperty');
        else
            id_underscore = max(id_underscore);
        end
        idx = propname((id_underscore + 1):end);   % The figure of the propname
        idx = str2double(idx);
        nb_lev = number_levels(ar_cov);
        if (idx <= 0) || (idx > nb_lev)
            stk_error('Wrong index number', 'InvalidArgument');
        end
            
        propname = propname(1:(id_underscore - 1));   % The name of the property
        params = ar_cov.params;
        if ~strcmp(propname, 'param')
            param_i = set(params{idx}, propname, value);
        else
            param_i = value;
        end
        
        if ~isa(param_i, 'stk_param')
            stk_error('New parameters must be a stk_param', 'InvalidArgument');
        end
        params{idx} = param_i;
        
        ar_cov.params = params;
end

end

