% CALL : par_bro = set(par_bro, propname, value)
% [overload stk_param function]
function par_bro = set(par_bro, propname, value)

switch propname
    case 'logsig2'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''logsig2'' must be a double.', 'InvalidType');
            end
        end
        % Assert size
        if numel(value) > 1
            stk_error('''logsig2'' must have a unique value.', 'IncorrectSize');
        end
        % Change value
        par_bro.logsig2 = value;
        
    case 'logdegr'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''logdegr'' must be a double.', 'InvalidType');
            end
        end
        % Assert size
        if numel(value) > 1
            stk_error('''logdegr'' must have a unique value.', 'IncorrectSize');
        end
        % Change value
        par_bro.logdegr = value;
        
    case 'intercept'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''intercept'' must be a double.', 'InvalidType');
            end
        end
        % Assert size
        if numel(value) > 1
            stk_error('''intercept'' must have a unique value.', 'IncorrectSize');
        end
        % Change value
        par_bro.intercept = value;
        
        %% Other supported changes
    case 'sig2'
        par_bro = set(par_bro, 'logsig2', reallog(value));
        % reallog to ensure that input is positive
    case 'degr'
        par_bro = set(par_bro, 'logdegr', reallog(value));
        
    case 'logsig'
        par_bro = set(par_bro, 'logsig2', 2*value);
    case 'sig'
        par_bro = set(par_bro, 'logsig2', 2*reallog(value));
        
    otherwise
        stk_error(['An brownian parameter does not have any property',...
            ' whose name is ', propname, '.'], 'UnknownField');
end

end