% CALL : [lb, ub] = stk_param_getdefaultbounds(par_iso, xi, zi)
%
% Get the lower bound and upper bound of optimization for a parameter.
% lb <= param <= ub.
% lb and ub are vector with the same length as param.
function [lb, ub] = stk_param_getdefaultbounds (par_iso, xi, zi)

if nargin > 3
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% constants
opts = stk_options_get ('stk_param_getdefaultbounds');
TOLVAR = opts.tolvar;
TOLSCALE = opts.tolscale;

%% Bounds for the variance parameter
log_empirical_variance = log (var (double (zi)));
logsig2_mid = par_iso.logsig2;
if isinf(log_empirical_variance) && log_empirical_variance < 0 % == - Inf
    logvar_lb = logsig2_mid - TOLVAR;
    logvar_ub = logsig2_mid + TOLVAR;
else
    logvar_lb = min (log_empirical_variance, logsig2_mid) - TOLVAR;
    logvar_ub = max (log_empirical_variance, logsig2_mid) + TOLVAR;
end

%% Bounds of other parameters
range_mid = par_iso.logacc;
range_lb  = range_mid(:) - TOLSCALE;
range_ub  = range_mid(:) + TOLSCALE;

lb = [logvar_lb; range_lb;];
ub = [logvar_ub; range_ub;];

end

