% CALL : par_iso = stk_param_iso(logsig2, logacc)
%
% Build an isometric parameter.
% This parameter should be used for isometric covariance :
%   k(x, y) = sig2*c( ||x - y||*acc ).
% - logsig2 : logarithm of the variance at 0 of the covariance. Must be a
% unique numeric;
% - logacc : logarithm of the accuracy of the covariance. Accuracy is the
% inverse of range. Must be a unique numeric;
function par_iso = stk_param_iso(logsig2, logacc)

if nargin > 2
    stk_error('Too many input argument.', 'TooManyInputArgs');
end

%% Accuracy
if nargin < 2 || isempty(logacc)
    logacc = NaN;    % Default value : no value
else
    % In contrast, accuracy must be a unique double.
    if ~isnumeric(logacc)
        try
            logacc = double(logacc);
        catch
            stk_error('''logacc'' must be a double.', 'InvalidType');
        end
    end

    if numel(logacc) > 1
        stk_error('There should be only one value for ''logacc''.', 'IncorrectSize');
    end
end

%% Variance at 0
if nargin < 1 || isempty(logsig2)
    logsig2 = NaN;    % Default value : no value
else
    % Variance at 0 must be a unique double too.
    if ~isnumeric(logsig2)
        try
            logsig2 = double(logsig2);
        catch
            stk_error('''logsig2'' must be a double.', 'InvalidType');
        end
    end

    if numel(logsig2) > 1
        stk_error('There should be only one value for ''logsig2''.', 'IncorrectSize');
    end
end

%% Build the isometric parameter
par_iso = struct('logsig2', logsig2, 'logacc', logacc);
par_iso = class(par_iso, 'stk_param_iso', stk_param());

end

