% CALL : mat_aniso = stk_param_matern_aniso(logsig2, logacc, logreg)
%
% Build an anisometric Matern parameter.
% This parameter should be used for anisometric Matern covariance :
%   k(x, y) = sig2*M{reg}( ||(x - y).*acc|| ).
% - logsig2 : logarithm of the variance at 0 of the covariance. Must be a
% unique numeric;
% - logacc : logarithm of the accuracies of the covariance. Accuracies are
% the inverse of ranges. Must be a vector;
% - logreg : logarithm of the Matern covariance. Must be a unique numeric;
function mat_aniso = stk_param_matern_aniso(logsig2, logacc, logreg)


%% Regularity
if nargin < 3 || isempty(logreg)
    logreg = NaN;    % Default value : no value
else
    % In contrast, regularity must be a unique double.
    if ~isnumeric(logreg)
        try
            logreg = double(logreg);
        catch
            stk_error('''logreg'' must be a double.', 'InvalidType');
        end
    end

    if numel(logreg) > 1
        stk_error('There should be only one value for ''logreg''.', 'IncorrectSize');
    end
end

%% A matern anisometric parameter is an anisometric parameter
if nargin < 2
    logacc = [];
    if nargin < 1
        logsig2 = [];
    end
end
par_aniso = stk_param_aniso(logsig2, logacc);

%% Build the isotropic Matern covariance
mat_aniso = struct('logreg', logreg);
mat_aniso = class(mat_aniso, 'stk_param_matern_aniso', par_aniso);
end

