% CALL : mat_iso = stk_param_matern_iso(logsig2, logacc, logreg)
%
% Build an isometric Matern parameter.
% This parameter should be used for isometric Matern covariance :
%   k(x, y) = sig2*M{reg}( ||x - y||*acc ).
% - logsig2 : logarithm of the variance at 0 of the covariance. Must be a
% unique numeric;
% - logacc : logarithm of the accuracy of the covariance. Accuracy is the
% inverse of range. Must be a unique numeric;
% - logreg : logarithm of the Matern covariance. Must be a unique numeric;
function mat_iso = stk_param_matern_iso(logsig2, logacc, logreg)

%% Regularity
if nargin < 3 || isempty(logreg)
    logreg = NaN;    % Default value : no value
else
    % In contrast, accuracy must be a unique double.
    if ~isnumeric(logreg)
        try
            logreg = double(logreg);
        catch
            stk_error('''logreg'' must be a double.', 'InvalidType');
        end
    end

    if numel(logreg) > 1
        stk_error('There should be only one value for ''logreg''.', 'IncorrectSize');
    end
end
 
%% A matern isometric parameter is an isometric parameter
if nargin < 2
    logacc = [];
    if nargin < 1
        logsig2 = [];
    end
end
par_iso = stk_param_iso(logsig2, logacc);

%% Build the isotropic Matern covariance
mat_iso = struct('logreg', logreg);
mat_iso = class(mat_iso, 'stk_param_matern_iso', par_iso);
end

