% CALL : mf_lnv = set(mf_lnv, propname, value)
% [overload stk_param_cstlnv function]
function mf_lnv = set(mf_lnv, propname, value)

switch propname
    case 'levels'
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error('''levels'' must be a double.', 'InvalidType');
            end
        end
        value = value(:);                   % assert vector
        value = unique(value, 'stable');    % assert unique value
        mf_lnv.levels = value;              % change value
        
        % Check that lognoisevar and levels have the same size.
        lognoivar = get(mf_lnv, 'lognoivar');
        
        if length(value) < length(lognoivar)
            % if there are less levels than log-noise variances
            if any(~isnan(lognoivar))
                % if there is at least one non-NaN value.
                warning('STK:set:autoDeleteLevels',...
                    'To follow the number of levels, some log-noise variances were deleted.');
            end
            mf_lnv = set(mf_lnv, 'lognoivar', lognoivar(1:length(value)));
        elseif length(value) > length(lognoivar)
            % if there are more levels than log-noise variances
            if any(~isnan(lognoivar))
                % if there is at least one non-NaN value.
                warning('STK:set:autoAddNaNLevels',...
                    'To follow the number of levels, some log-noise variances were added.');
            end
            mf_lnv = set(mf_lnv, 'lognoivar', [lognoivar;
                NaN(length(value) - length(lognoivar), 1)]);
        end % else if there are the same numbers ==> all is OK
        
    case {'lognoivar', 'lognoistd', 'noivar', 'noistd'}
        % Assert class
        if ~isnumeric(value)
            try
                value = double(value);
            catch
                stk_error(['''', propname, ''' must be a double.'], 'InvalidType');
            end
        end
        value = value(:);   % assert vector
        % Assert size
        if length(value) ~= length(mf_lnv.levels)
            stk_error(['''', propname,...
                ''' must have the same length than the levels.'], 'IncorrectSize');
        end
        % Change value
        mf_lnv.stk_param_cstlnv = set(mf_lnv.stk_param_cstlnv, propname, value);
        
    otherwise
        mf_lnv.stk_param_cstlnv = set(mf_lnv.stk_param_cstlnv, propname, value);
end
end

