% CALL : [lb, ub] = stk_param_getdefaultbounds(nonstat_cov, xi, zi)
%
% Get the lower bound and upper bound of optimization for a parameter.
% lb <= param <= ub.
% lb and ub are vector with the same length as param.
function [lb, ub] = stk_param_getdefaultbounds(nonstat_cov, xi, zi)


if nargin > 3
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% Check size
if size(xi, 1) ~= size(zi, 1);
    stk_error('xi and zi does not have the same size.', 'IncorrectSize');
end

if size(zi, 2) > 1
    stk_error('zi must be a vector.', 'IncorrectSize');
end

ti = double(xi(:, end));
zi = double(zi);

%% Bounds for the variance parameter
[par_lb, par_ub] = stk_param_getdefaultbounds(nonstat_cov.param, xi, zi);

%% Bounds of the intercept
intercept = nonstat_cov.intercept;
if isnan(intercept) || isinf(intercept)
    intercept = 0;
end
inter_lb = intercept;
inter_ub = intercept; % No optimization according to intercept
%% Bounds for the degree

logdeg_lb = -5; % ~= 0.0067, no decreasing
logdeg_ub = 5;  % ~= 150, fast decreasing

logdegr = nonstat_cov.logdegr;

if isnan(logdegr) || isinf(logdegr)
    logdeg_lb = min(logdegr - 2.5, logdeg_lb);
    logdeg_ub = max(logdegr + 2.5, logdeg_ub);
end
% exp(2.5) ~= 12 => degree in [par_bro.logdegr/12; par_bro.logdegr*12]

%% Return bounds
lb = [par_lb; logdeg_lb; inter_lb];
ub = [par_ub; logdeg_ub; inter_ub];

end

