% CALL : value = get (gauss_lp, propname)
% [overload stk_prior function]
function value = get (gauss_lp, propname)

switch propname
    case 'mean'
        value = gauss_lp.mean;
    case 'eigenvals'
        value = gauss_lp.eigenvals;
    case 'eigenvect'
        value = gauss_lp.eigenvect;
        %% Other values returned by get function
    case 'covariance'
        D = gauss_lp.eigenvals;
        D(isinf(D)) = 1i;	% replace infinite values by 1i, to make multiplication
        D = diag(D);
        P = gauss_lp.eigenvect;
        value = P*D*P';
        
        % If a value has an imaginary part, that means that this value is
        % infinite, in reality.
        ind = abs(imag(value(:))) < 1e-8;
        value(ind) = real(value(ind));      % remove negligeable imaginary part
        imag_to_inf = imag(value(~ind));
        value(~ind) = sign(imag_to_inf).*Inf(size(imag_to_inf));
        % change value : 1i*x, by +Inf*x, with +Inf*0 = 0
    case 'invcov'
        D = 1./gauss_lp.eigenvals;
        D(isinf(D)) = 1i;   % replace infinite values by 1i, to make multiplication
        D = diag(D);
        P = gauss_lp.eigenvect;
        value = P*D*P';
        
        % If a value has an imaginary part, that means that this value is
        % infinite, in reality.
        ind = abs(imag(value(:))) < 1e-8;
        value(ind) = real(value(ind));  % remove negligeable imaginary part
        imag_to_inf = imag(value(~ind));
        value(~ind) = sign(imag_to_inf).*Inf(size(imag_to_inf));
        % change value : 1i*x, by +Inf*x, with +Inf*0 = 0
    case 'variance'
        value = diag(get(gauss_lp, 'covariance'));
        
    case 'invvar'
        value = diag(get(gauss_lp, 'invcov'));
        
    case 'std'
        value = sqrt(get(gauss_lp, 'variance'));
        
    otherwise
        stk_error(['A gaussian log-prior does not have any property',...
            ' whose name is ', propname, '.'], 'UnknownField');
end
end

