% CALL : param = stk_generate_prior_param(gauss_lp, n)
%
% Generate n parameters according to Gaussian log-prior distribution.
%
% - gauss_lp : an stk_prior_gauss object;
% - n : an integer;
%
% * param : a matrix of size n x d, where d is the dimension of gauss_lp.
function param = stk_generate_prior_param(gauss_lp, n)

if nargin > 2
    stk_error('Too many input arguments.', 'TooManyInputArgs');
end

%% Default value for n
if nargin < 2 || isempty(n)
    n = 1;
end

d = dimension(gauss_lp);
%% Get parameters
eigenvar = gauss_lp.eigenvals;
if any(isinf(eigenvar))
    stk_error(['Impossible to simulate parameters whose one of eigen values',...
        ' is infinite. Replace it by a large number.'], 'matrixWithInf');
end
eigVectors = gauss_lp.eigenvect;
mean_vect = gauss_lp.mean;

%% Generate param according to N(0, 1)
pa = randn(n, d);

%% Add covariance
para = pa*diag(sqrt(eigenvar))*eigVectors';

%% Add mean
param = para + repmat(mean_vect', n, 1);
end

